@@ -44,8 +44,8 @@ struct _EBookBackendGroupwisePrivate {
 	gboolean is_writable;
 	char *summary_file_name;
 	EBookBackendSummary *summary;
-	char *old_name;
-	gboolean to_be_renamed;
+	gboolean is_summary_ready;
+	char *use_ssl;
 };
 
 #define ELEMENT_TYPE_SIMPLE 0x01
@@ -455,10 +455,10 @@ static void
 populate_birth_date (EContact *contact, gpointer data)
 {
 	EGwItem *item;
-  
-	item = E_GW_ITEM (data);
 	char *value ;
 	EContactDate *date;
+  
+	item = E_GW_ITEM (data);
 	value = e_gw_item_get_field_value (item, "birthday");
  	if (value) {
 		date =  e_contact_date_from_string (value);
@@ -539,9 +539,9 @@ set_emails_in_gw_item (EGwItem *item, gpointer data)
 	GList *email_list;
 	EContact *contact;
 	char *email;
-	contact = E_CONTACT (data);
 	int i;
 
+	contact = E_CONTACT (data);
 	email_list = NULL;
 	for (i =0 ; i < 3; i++) {
 		email = e_contact_get (contact, email_fields[i]);
@@ -609,14 +609,13 @@ static void
 populate_full_name (EContact *contact, gpointer data)
 {
 	EGwItem *item;
-	item = E_GW_ITEM(data);
 	FullName  *full_name ;
 	char *full_name_string;
 
+	item = E_GW_ITEM(data);
 	full_name = e_gw_item_get_full_name (item);
 	if (full_name) {
-		full_name_string = g_strconcat ( (full_name->name_prefix == NULL) ? "\0" : full_name->name_prefix, " ",
-			    (full_name->first_name == NULL) ? "\0" :    full_name->first_name, " ",
+		full_name_string = g_strconcat ( (full_name->first_name == NULL) ? "\0" :    full_name->first_name, " ",
 			    (full_name->middle_name == NULL) ? "\0" : full_name->middle_name, " ",
 			    full_name->last_name == NULL ? "\0" : full_name->last_name, " ",
 			    (full_name->name_suffix == NULL ) ? "\0" : full_name->name_suffix, NULL);
@@ -806,7 +805,7 @@ set_members_in_gw_item (EGwItem  *item, EContact *contact, EBookBackendGroupwise
 	e_gw_filter_group_conditions (filter, E_GW_FILTER_OP_OR, count);
 	items = NULL;
 	if (count)
-		status = e_gw_connection_get_items (egwb->priv->cnc, egwb->priv->container_id, NULL, filter, &items);
+		status = e_gw_connection_get_items (egwb->priv->cnc, egwb->priv->container_id, "members", filter, &items);
 	for (; items != NULL; items = g_list_next (items )) {
 		GList *emails;
 		GList *ptr;
@@ -1099,7 +1098,7 @@ e_book_backend_groupwise_create_contact (EBookBackend *backend,
 	if (egwb->priv->cnc == NULL) {
 		e_data_book_respond_create(book, opid, GNOME_Evolution_Addressbook_AuthenticationRequired, NULL);
 	}
-       
+       	
 	contact = e_contact_new_from_vcard(vcard);
 	item = e_gw_item_new_empty ();
 	e_gw_item_set_item_type (item, e_contact_get (contact, E_CONTACT_IS_LIST) ? E_GW_ITEM_TYPE_GROUP :E_GW_ITEM_TYPE_CONTACT);
@@ -1341,6 +1340,8 @@ typedef struct {
 	EGwFilter *filter;
 	gboolean is_filter_valid;
 	gboolean is_personal_book;
+	int auto_completion; 
+	char *search_string;
 } EBookBackendGroupwiseSExpData;
 
 static ESExpResult *
@@ -1428,10 +1429,8 @@ func_contains(struct _ESExp *f, int argc, struct _ESExpResult **argv, void *data
 				else 
 					e_gw_filter_group_conditions (filter, E_GW_FILTER_OP_OR, 2);
 			}
-			else if (sexp_data->is_personal_book)
-				e_gw_filter_add_filter_component (filter, E_GW_FILTER_OP_CONTAINS, gw_field_name, str);
 			else 
-				sexp_data->is_filter_valid = FALSE;
+				e_gw_filter_add_filter_component (filter, E_GW_FILTER_OP_CONTAINS, gw_field_name, str);
 		}
 		else 
 		     sexp_data->is_filter_valid = FALSE; 
@@ -1479,10 +1478,9 @@ func_is(struct _ESExp *f, int argc, struct _ESExpResult **argv, void *data)
 				else 
 					e_gw_filter_group_conditions (filter, E_GW_FILTER_OP_OR, 2);
 			}
-			else if (sexp_data->is_personal_book)
-				e_gw_filter_add_filter_component (filter, E_GW_FILTER_OP_EQUAL, gw_field_name, str);
 			else 
-				sexp_data->is_filter_valid = FALSE;
+				e_gw_filter_add_filter_component (filter, E_GW_FILTER_OP_EQUAL, gw_field_name, str);
+		
 		}
 		else 
 		     sexp_data->is_filter_valid = FALSE;
@@ -1495,6 +1493,12 @@ func_is(struct _ESExp *f, int argc, struct _ESExpResult **argv, void *data)
 	return r;
 }
 
+#define BEGINS_WITH_NAME (1 << 0)
+#define BEGINS_WITH_EMAIL (1 << 1)
+#define BEGINS_WITH_FILE_AS (1 << 2)
+#define BEGINS_WITH_NICK_NAME (1 << 3)
+#define AUTO_COMPLETION_QUERY 15
+
 static ESExpResult *
 func_beginswith(struct _ESExp *f, int argc, struct _ESExpResult **argv, void *data)
 {
@@ -1514,12 +1518,23 @@ func_beginswith(struct _ESExp *f, int argc, struct _ESExpResult **argv, void *da
 		char *gw_field_name;
 	
 		gw_field_name = NULL;
-		if (g_str_equal (propname, "full_name"))
+		if (g_str_equal (propname, "full_name")) {
 			gw_field_name = "fullName";
-		else if (g_str_equal (propname, "email"))
+			sexp_data->auto_completion |= BEGINS_WITH_NAME;
+			sexp_data->search_string = g_strdup (str);
+		}
+		else if (g_str_equal (propname, "email")) {
 			gw_field_name = "emailList/email";
-		else if (g_str_equal (propname, "file_as") || g_str_equal (propname, "nickname"))
+			sexp_data->auto_completion |= BEGINS_WITH_EMAIL;
+		}
+		else if (g_str_equal (propname, "file_as")) { 
 			 gw_field_name = "name";
+			 sexp_data->auto_completion |= BEGINS_WITH_FILE_AS;
+		} else if (g_str_equal (propname, "nickname")) { 
+			 gw_field_name = "name";
+			 sexp_data->auto_completion |= BEGINS_WITH_NICK_NAME;
+		}
+
 		if (gw_field_name) {
 			
 			if (g_str_equal (gw_field_name, "fullName")) {
@@ -1532,10 +1547,9 @@ func_beginswith(struct _ESExp *f, int argc, struct _ESExpResult **argv, void *da
 				else 
 					e_gw_filter_group_conditions (filter, E_GW_FILTER_OP_OR, 2);
 			}
-			else if (sexp_data->is_personal_book)
-				e_gw_filter_add_filter_component (filter, E_GW_FILTER_OP_BEGINS, gw_field_name, str);
 			else 
-				sexp_data->is_filter_valid = FALSE;
+				e_gw_filter_add_filter_component (filter, E_GW_FILTER_OP_BEGINS, gw_field_name, str);
+			
 		}
 		else 
 			sexp_data->is_filter_valid = FALSE;
@@ -1602,10 +1616,8 @@ func_exists(struct _ESExp *f, int argc, struct _ESExpResult **argv, void *data)
 				else 
 					e_gw_filter_group_conditions (filter, E_GW_FILTER_OP_OR, 2);
 			}
-			else if (sexp_data->is_personal_book)
-				e_gw_filter_add_filter_component (filter, E_GW_FILTER_OP_EXISTS, gw_field_name, str);
 			else 
-				sexp_data->is_filter_valid = FALSE;
+				e_gw_filter_add_filter_component (filter, E_GW_FILTER_OP_EXISTS, gw_field_name, str);
 		}
 		else 
 			sexp_data->is_filter_valid = FALSE;
@@ -1636,7 +1648,7 @@ static struct {
 
 
 static EGwFilter*
-e_book_backend_groupwise_build_gw_filter (EBookBackendGroupwise *ebgw, const char *query )
+e_book_backend_groupwise_build_gw_filter (EBookBackendGroupwise *ebgw, const char *query, gpointer is_auto_completion, char ** search_string)
 {
 	ESExp *sexp;
 	ESExpResult *r;
@@ -1652,6 +1664,8 @@ e_book_backend_groupwise_build_gw_filter (EBookBackendGroupwise *ebgw, const cha
 	sexp_data->filter = filter;
 	sexp_data->is_filter_valid = TRUE;
 	sexp_data->is_personal_book = e_book_backend_is_writable ( E_BOOK_BACKEND (ebgw));
+	sexp_data->auto_completion = 0;
+	sexp_data->search_string = NULL;
 
 	for(i=0;i<sizeof(symbols)/sizeof(symbols[0]);i++) {
 		if (symbols[i].type == 1) {
@@ -1671,6 +1685,10 @@ e_book_backend_groupwise_build_gw_filter (EBookBackendGroupwise *ebgw, const cha
 	
 	
 	if (sexp_data->is_filter_valid) {
+		if (sexp_data->auto_completion == AUTO_COMPLETION_QUERY)
+			*(gboolean *)is_auto_completion = TRUE;
+		if (search_string)
+			*search_string = sexp_data->search_string;
 		g_free (sexp_data);
 		return filter;
 	}
@@ -1698,9 +1716,10 @@ e_book_backend_groupwise_get_contact_list (EBookBackend *backend,
 	EBookBackendGroupwise *egwb;
 	gboolean match_needed;
 	EBookBackendSExp *card_sexp = NULL;
-	EGwFilter *filter;
+	EGwFilter *filter = NULL;
 	GPtrArray *ids;
-
+	gboolean is_auto_completion;
+	
 	egwb = E_BOOK_BACKEND_GROUPWISE (backend);
 	vcard_list = NULL;
 	gw_items = NULL;
@@ -1711,9 +1730,6 @@ e_book_backend_groupwise_get_contact_list (EBookBackend *backend,
 	}
 	
 	match_needed = TRUE;
-	filter = e_book_backend_groupwise_build_gw_filter (egwb, query);
-	if (filter)
-		match_needed = FALSE; 
 	card_sexp = e_book_backend_sexp_new (query);
 	if (!card_sexp) {
 		e_data_book_respond_get_contact_list (book, opid, GNOME_Evolution_Addressbook_InvalidQuery,
@@ -1721,19 +1737,19 @@ e_book_backend_groupwise_get_contact_list (EBookBackend *backend,
 	}
 
 	status = E_GW_CONNECTION_STATUS_OK;
-	if (e_book_backend_summary_is_summary_query (egwb->priv->summary, query)) {
+	if (egwb->priv->is_summary_ready && e_book_backend_summary_is_summary_query (egwb->priv->summary, query)) {
 	
 		ids = e_book_backend_summary_search (egwb->priv->summary, query);
 		if (ids->len > 0)
-			status = e_gw_connection_get_items_from_ids (egwb->priv->cnc, egwb->priv->container_id, NULL, ids, &gw_items);
+			status = e_gw_connection_get_items_from_ids (egwb->priv->cnc, egwb->priv->container_id, "members", ids, &gw_items);
 		match_needed = FALSE;
 		g_ptr_array_free (ids, TRUE);
 	} else { 
 		if (strcmp (query, "(contains \"x-evolution-any-field\" \"\")") != 0)
-			filter = e_book_backend_groupwise_build_gw_filter (egwb, query);
+			filter = e_book_backend_groupwise_build_gw_filter (egwb, query, &is_auto_completion, NULL);
 		if (filter)
 			match_needed = FALSE;
-		status = e_gw_connection_get_items (egwb->priv->cnc, egwb->priv->container_id, NULL, filter, &gw_items);
+		status = e_gw_connection_get_items (egwb->priv->cnc, egwb->priv->container_id, "members", filter, &gw_items);
 	}
 
 	if (status != E_GW_CONNECTION_STATUS_OK) {
@@ -1744,8 +1760,10 @@ e_book_backend_groupwise_get_contact_list (EBookBackend *backend,
 	for (; gw_items != NULL; gw_items = g_list_next(gw_items)) { 
 		contact = e_contact_new ();
 		fill_contact_from_gw_item (contact, E_GW_ITEM (gw_items->data), egwb->priv->categories_by_id);
-		if (match_needed &&  e_book_backend_sexp_match_contact (card_sexp, contact))
-			vcard_list = g_list_append (vcard_list, e_vcard_to_string (E_VCARD (contact), EVC_FORMAT_VCARD_30));
+		if (match_needed) {  
+			if (e_book_backend_sexp_match_contact (card_sexp, contact))
+				vcard_list = g_list_append (vcard_list, e_vcard_to_string (E_VCARD (contact), EVC_FORMAT_VCARD_30));
+		}
 		else 
 			vcard_list = g_list_append (vcard_list, e_vcard_to_string (E_VCARD (contact), EVC_FORMAT_VCARD_30));
 		g_object_unref (contact);
@@ -1808,37 +1826,66 @@ book_view_thread (gpointer data)
 	const char *query;
 	EGwFilter *filter = NULL;
 	GPtrArray *ids;
-
 	gboolean stopped = FALSE;
 	EDataBookView *book_view = data;
 	GroupwiseBackendSearchClosure *closure = get_closure (book_view);
+	char *view = NULL;
+	gboolean is_auto_completion = FALSE;
+	char *search_string = NULL;
 
 	gwb  = closure->bg;
 	gw_items = NULL;
 
+
 	bonobo_object_ref (book_view);
+	g_mutex_lock (closure->mutex);
+	g_cond_signal (closure->cond);
+	g_mutex_unlock (closure->mutex);
+
 	if (gwb->priv->cnc == NULL) {
 		e_data_book_view_notify_complete (book_view, GNOME_Evolution_Addressbook_AuthenticationRequired);
 		bonobo_object_unref (book_view);
 		return NULL; 
 	}
-	e_data_book_view_notify_status_message (book_view, _("Searching..."));
-	g_mutex_lock (closure->mutex);
-	g_cond_signal (closure->cond);
-	g_mutex_unlock (closure->mutex);
+
+
 	query = e_data_book_view_get_card_query (book_view);
+
+	filter = e_book_backend_groupwise_build_gw_filter (gwb, query, &is_auto_completion, &search_string);
+	view = "members";
+	if (is_auto_completion) 
+		view = "name email";
+	if (is_auto_completion && !gwb->priv->is_writable) {
+		g_object_unref (filter);
+		filter = NULL;
+
+		if (search_string) {
+			filter = e_gw_filter_new ();
+			e_gw_filter_add_filter_component (filter, E_GW_FILTER_OP_BEGINS, "fullName/lastName", search_string);
+			e_gw_filter_add_filter_component (filter, E_GW_FILTER_OP_BEGINS, "fullName/firstName", search_string);
+			e_gw_filter_group_conditions (filter, E_GW_FILTER_OP_OR, 2);
+			g_free (search_string);
+			
+		}
+	}
+
+	if (!gwb->priv->is_writable && !filter) {
+		e_data_book_view_notify_complete (book_view, GNOME_Evolution_Addressbook_Success);
+		bonobo_object_unref (book_view);
+		return NULL; 
+	}
 	
+	e_data_book_view_notify_status_message (book_view, _("Searching..."));
 	status =  E_GW_CONNECTION_STATUS_OK;
-	if (e_book_backend_summary_is_summary_query (gwb->priv->summary, query)) {
+	if (gwb->priv->is_summary_ready && e_book_backend_summary_is_summary_query (gwb->priv->summary, query)) {
 	
 		ids = e_book_backend_summary_search (gwb->priv->summary, query);
-		if (ids->len > 0)
-			status = e_gw_connection_get_items_from_ids (gwb->priv->cnc, gwb->priv->container_id, NULL, ids, &gw_items);
+		if (ids->len > 0) 
+			status = e_gw_connection_get_items_from_ids (gwb->priv->cnc, gwb->priv->container_id, view, ids, &gw_items);
 		g_ptr_array_free (ids, TRUE);
 	} else { 
-		if (strcmp (query, "(contains \"x-evolution-any-field\" \"\")") != 0)
-			filter = e_book_backend_groupwise_build_gw_filter (gwb, query);
-		status = e_gw_connection_get_items (gwb->priv->cnc, gwb->priv->container_id, NULL, filter, &gw_items);
+		
+		status = e_gw_connection_get_items (gwb->priv->cnc, gwb->priv->container_id, view, filter, &gw_items);
 	}
 	
 	if (status != E_GW_CONNECTION_STATUS_OK) {
@@ -1860,7 +1907,11 @@ book_view_thread (gpointer data)
 		}
 		contact = e_contact_new ();
 		fill_contact_from_gw_item (contact, E_GW_ITEM (gw_items->data), gwb->priv->categories_by_id);
-		e_data_book_view_notify_update (book_view, contact);
+		if (e_contact_get_const (contact, E_CONTACT_UID)) 
+			e_data_book_view_notify_update (book_view, contact);
+		else 
+			g_critical ("Id missing for item %s\n", e_contact_get_const (contact, E_CONTACT_FILE_AS));
+
 		g_object_unref(contact);
 		g_object_unref (gw_items->data);
 		
@@ -1880,10 +1931,11 @@ e_book_backend_groupwise_start_book_view (EBookBackend  *backend,
 				     EDataBookView *book_view)
 {
 	GroupwiseBackendSearchClosure *closure = init_closure (book_view, E_BOOK_BACKEND_GROUPWISE (backend));
-       	g_mutex_lock (closure->mutex);
-	closure->thread = g_thread_create (book_view_thread, book_view, TRUE, NULL);
+	
+	g_mutex_lock (closure->mutex);
+	closure->thread = g_thread_create (book_view_thread, book_view, FALSE, NULL);
 	g_cond_wait (closure->cond, closure->mutex);
-
+	
 	/* at this point we know the book view thread is actually running */
 	g_mutex_unlock (closure->mutex);
 }
@@ -1893,16 +1945,13 @@ e_book_backend_groupwise_stop_book_view (EBookBackend  *backend,
 					 EDataBookView *book_view)
 {
 	GroupwiseBackendSearchClosure *closure = get_closure (book_view);
-	gboolean need_join = FALSE;
+	
 
 	g_mutex_lock (closure->mutex);
 	if (!closure->stopped)
-		need_join = TRUE;
-	closure->stopped = TRUE;
+		closure->stopped = TRUE;
 	g_mutex_unlock (closure->mutex);
 	
-	if (need_join)
-		g_thread_join (closure->thread);
 }
 
 static void
@@ -1923,9 +1972,11 @@ build_summary (EBookBackendGroupwise *ebgw)
 	int status;
 	GList *gw_items = NULL;
 	EContact *contact;
-	status = e_gw_connection_get_items (ebgw->priv->cnc, ebgw->priv->container_id, NULL, NULL, &gw_items);
-	if (status != E_GW_CONNECTION_STATUS_OK)
+	status = e_gw_connection_get_items (ebgw->priv->cnc, ebgw->priv->container_id, "members", NULL, &gw_items);
+	if (status != E_GW_CONNECTION_STATUS_OK) 
 		return FALSE;
+
+
 	for (; gw_items != NULL; gw_items = g_list_next(gw_items)) { 
 		contact = e_contact_new ();
 		fill_contact_from_gw_item (contact, E_GW_ITEM (gw_items->data), ebgw->priv->categories_by_id);
@@ -1934,6 +1985,7 @@ build_summary (EBookBackendGroupwise *ebgw)
 		g_object_unref (gw_items->data);
 		
 	}
+	ebgw->priv->is_summary_ready = TRUE;
 	g_list_free (gw_items);
 	return FALSE;
 }
@@ -1957,9 +2009,10 @@ update_summary (EBookBackendGroupwise *ebgw)
 
 	filter = e_gw_filter_new ();
 	e_gw_filter_add_filter_component (filter, E_GW_FILTER_OP_GREATERTHAN, "modified", time_string);
-	status = e_gw_connection_get_items (ebgw->priv->cnc, ebgw->priv->container_id, NULL, filter, &gw_items);
+	status = e_gw_connection_get_items (ebgw->priv->cnc, ebgw->priv->container_id, "members", filter, &gw_items);
 	if (status != E_GW_CONNECTION_STATUS_OK)
 		return FALSE;
+	
 	for (; gw_items != NULL; gw_items = g_list_next(gw_items)) { 
 		char *id;
 		contact = e_contact_new ();
@@ -1976,6 +2029,7 @@ update_summary (EBookBackendGroupwise *ebgw)
 		
 		    
 	}
+	ebgw->priv->is_summary_ready = TRUE;
 	g_object_unref (filter);
 	g_list_free (gw_items);
 	return FALSE;
@@ -1993,16 +2047,29 @@ e_book_backend_groupwise_authenticate_user (EBookBackend *backend,
 	EBookBackendGroupwisePrivate *priv;
 	char *id;
 	int status;
+	char *http_uri;
 	gboolean is_writable;
+	
 
 	ebgw = E_BOOK_BACKEND_GROUPWISE (backend);
 	priv = ebgw->priv;
   
+	if (priv->cnc) { /*we have already authenticated to server */
+		e_data_book_respond_authenticate_user (book, opid, GNOME_Evolution_Addressbook_Success); 
+		return;
+	}
+
 	priv->cnc = e_gw_connection_new (priv->uri, user, passwd);
-	if (priv->cnc == NULL) {
+	if (!E_IS_GW_CONNECTION(priv->cnc) && priv->use_ssl && g_str_equal (priv->use_ssl, "when-possible")) {
+		http_uri = g_strconcat ("http://", priv->uri + 8, NULL);
+		priv->cnc = e_gw_connection_new (http_uri, user, passwd);
+		g_free (http_uri);
+	}
+	if (!E_IS_GW_CONNECTION(priv->cnc)) {
 		e_data_book_respond_authenticate_user (book, opid, GNOME_Evolution_Addressbook_AuthenticationFailed);
 		return;
 	}
+	
 	id = NULL;
 	is_writable = FALSE;
 	status = e_gw_connection_get_address_book_id (priv->cnc,  priv->book_name, &id, &is_writable); 
@@ -2035,9 +2102,13 @@ e_book_backend_groupwise_authenticate_user (EBookBackend *backend,
              
 	e_util_mkdir_hier (g_path_get_dirname (priv->summary_file_name), 0700);
 	priv->summary = e_book_backend_summary_new (priv->summary_file_name, 5000);
-       	if (e_book_backend_summary_load (priv->summary) == FALSE)
-		g_idle_add ((GSourceFunc)build_summary ,ebgw);  
-	g_idle_add ((GSourceFunc)update_summary ,ebgw);
+       	
+	if (priv->is_writable) { /* no summaries for system address book*/
+		if(e_book_backend_summary_load (priv->summary) == FALSE)
+			g_idle_add ((GSourceFunc)build_summary ,ebgw);  
+		else 
+			g_idle_add ((GSourceFunc)update_summary ,ebgw);
+	}
 }
 
 static void
@@ -2083,6 +2154,7 @@ e_book_backend_groupwise_load_source (EBookBackend           *backend,
    	const char *port;
 	EUri *parsed_uri;
 	int i;
+	const char *use_ssl;
 
 	ebgw = E_BOOK_BACKEND_GROUPWISE (backend);
 	priv = ebgw->priv;
@@ -2102,8 +2174,12 @@ e_book_backend_groupwise_load_source (EBookBackend           *backend,
 	port = e_source_get_property (source, "port");
 	if (port == NULL)
 		port = "7181";
-       
-	priv->uri = g_strconcat ("http://", parsed_uri->host,":", port, "/soap", NULL );
+	use_ssl = e_source_get_property (source, "use_ssl");
+	if (use_ssl) 
+		priv->uri = g_strconcat ("https://", parsed_uri->host,":", port, "/soap", NULL );
+	else 
+		priv->uri = g_strconcat ("http://", parsed_uri->host,":", port, "/soap", NULL );
+	priv->use_ssl = g_strdup (use_ssl);
 	priv->only_if_exists = only_if_exists;
 	
 	
@@ -2135,9 +2211,13 @@ e_book_backend_groupwise_remove (EBookBackend *backend,
   
 	ebgw = E_BOOK_BACKEND_GROUPWISE (backend);
 	if (ebgw->priv->cnc == NULL) {
-		e_data_book_respond_remove (book,  opid, GNOME_Evolution_Addressbook_OtherError);
+		e_data_book_respond_remove (book,  opid,  GNOME_Evolution_Addressbook_AuthenticationRequired);
 		return;
-	}	
+	}
+	if (!ebgw->priv->is_writable) {
+		e_data_book_respond_remove (book,  opid,  GNOME_Evolution_Addressbook_PermissionDenied);
+		return;
+	}
 	status = e_gw_connection_remove_item (ebgw->priv->cnc, NULL, ebgw->priv->container_id);
 	if (status == E_GW_CONNECTION_STATUS_OK) 
 		e_data_book_respond_remove (book,  opid, GNOME_Evolution_Addressbook_Success);
@@ -2155,14 +2235,25 @@ e_book_backend_groupwise_get_static_capabilities (EBookBackend *backend)
   
 	ebgw = E_BOOK_BACKEND_GROUPWISE (backend);
 	if (ebgw->priv->is_writable)
-		return g_strdup("net,bulk-removes,do-initial-query");
+		return g_strdup("net,bulk-removes,do-initial-query,contact-lists");
 	else 
-		return g_strdup("net,bulk-removes");
+		return g_strdup("net,bulk-removes,contact-lists");
 }
 static void 
 e_book_backend_groupwise_get_supported_auth_methods (EBookBackend *backend, EDataBook *book, guint32 opid)
 {
-	/*FIXME  provide implementation*/  
+	GList *auth_methods = NULL;
+	char *auth_method;
+	
+	auth_method =  g_strdup_printf ("plain/password");
+	auth_methods = g_list_append (auth_methods, auth_method);
+	e_data_book_respond_get_supported_auth_methods (book,
+							opid,
+							GNOME_Evolution_Addressbook_Success,
+							auth_methods);  
+	g_free (auth_method);
+	g_list_free (auth_methods);
+	
 }
 
 
@@ -2216,9 +2307,13 @@ e_book_backend_groupwise_dispose (GObject *object)
 			bgw->priv->summary_file_name = NULL;
 		}
 		if (bgw->priv->summary) {
+			e_book_backend_summary_save(bgw->priv->summary);
 			g_object_unref (bgw->priv->summary);
 			bgw->priv->summary = NULL;
 		}
+		if (bgw->priv->use_ssl) {
+			g_free (bgw->priv->use_ssl);
+		}
 		g_free (bgw->priv);
 		bgw->priv = NULL;
 	}
@@ -2268,7 +2363,9 @@ e_book_backend_groupwise_init (EBookBackendGroupwise *backend)
 	priv->categories_by_id = g_hash_table_new_full (g_str_hash, g_str_equal, g_free, g_free);
 	priv->categories_by_name = g_hash_table_new_full (g_str_hash, g_str_equal, g_free, g_free);
 	priv->is_writable = TRUE;
-	backend->priv = priv;
+	priv->is_summary_ready = FALSE;
+	priv->use_ssl = NULL;
+       	backend->priv = priv;
 	
 }
 
