@@ -503,7 +503,6 @@ do_commit_contact (gboolean        sync,
 		g_cond_wait (our_op->cond, our_op->mutex);
 
 		status = our_op->status;
-		e_contact_set (contact, E_CONTACT_UID, our_op->id);
 		g_free (our_op->id);
 
 		/* remove the op from the book's hash of operations */
@@ -3060,6 +3059,7 @@ e_book_get_self (EContact **contact, EBook **book, GError **error)
 {
 	GError *e = NULL;
 	GConfClient *gconf;
+	gboolean status;
 	char *uid;
 
 	*book = e_book_new_system_addressbook (&e);
@@ -3070,6 +3070,13 @@ e_book_get_self (EContact **contact, EBook **book, GError **error)
 		return FALSE;
 	}
 
+	status = e_book_open (*book, FALSE, &e);
+	if (status == FALSE) {
+		if (error)
+			g_propagate_error (error, e);
+		return FALSE;
+	}
+
 	gconf = gconf_client_get_default();
 	uid = gconf_client_get_string (gconf, SELF_UID_KEY, NULL);
 	g_object_unref (gconf);
