@@ -20,6 +20,8 @@
  * Author: Chris Toshok (toshok@ximian.com)
  */
 
+#include <config.h>
+
 #include <glib.h>
 #include <stdio.h>
 #include <ctype.h>
@@ -33,7 +35,7 @@
 #define d(x)
 
 struct _EContactPrivate {
-	int padding;
+	char *cached_strings [E_CONTACT_FIELD_LAST];
 };
 
 #define E_CONTACT_FIELD_TYPE_STRING       0x00000001   /* used for simple single valued attributes */
@@ -254,9 +256,6 @@ e_contact_dispose (GObject *object)
 	EContact *ec = E_CONTACT (object);
 
 	if (ec->priv) {
-
-		/* XXX free instance specific stuff */
-
 		g_free (ec->priv);
 		ec->priv = NULL;
 	}
@@ -1281,12 +1280,49 @@ EContact*
 e_contact_new_from_vcard  (const char *vcard)
 {
 	EContact *contact;
+	const gchar *file_as;
 
 	g_return_val_if_fail (vcard != NULL, NULL);
 
 	contact = g_object_new (E_TYPE_CONTACT, NULL);
 	e_vcard_construct (E_VCARD (contact), vcard);
 
+	/* Generate a FILE_AS field if needed */
+
+	file_as = e_contact_get_const (contact, E_CONTACT_FILE_AS);
+	if (!file_as || !*file_as) {
+		EContactName *name;
+		const gchar *org;
+		gchar *file_as_new = NULL;
+		gchar *strings [4];
+		gchar **strings_p = strings;
+
+		name = e_contact_get (contact, E_CONTACT_NAME);
+		org = e_contact_get_const (contact, E_CONTACT_ORG);
+
+		if (name) {
+			if (name->family && *name->family)
+				*(strings_p++) = name->family;
+			if (name->given && *name->given)
+				*(strings_p++) = name->given;
+
+			if (strings_p != strings) {
+				*strings_p = NULL;
+				file_as_new = g_strjoinv (", ", strings);
+			}
+
+			e_contact_name_free (name);
+		}
+
+		if (!file_as_new && org && *org)
+			file_as_new = g_strdup (org);
+
+		if (file_as_new) {
+			e_contact_set (contact, E_CONTACT_FILE_AS, file_as_new);
+			g_free (file_as_new);
+		}
+	}
+
 	return contact;
 }
 
@@ -1328,6 +1364,11 @@ e_contact_pretty_name (EContactField field_id)
 
 	g_return_val_if_fail (field_id >= 1 && field_id <= E_CONTACT_FIELD_LAST, "");
 
+#ifdef ENABLE_NLS
+	bindtextdomain (GETTEXT_PACKAGE, EVOLUTION_LOCALEDIR);
+	bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");	
+#endif
+
 	for (i = 0; i < G_N_ELEMENTS (field_info); i ++) {
 		if (field_id == field_info[i].field_id)
 			return _(field_info[i].pretty_name);
@@ -1391,14 +1432,26 @@ free_const_data (gpointer data, GObject *where_object_was)
 const gpointer
 e_contact_get_const (EContact *contact, EContactField field_id)
 {
-	gpointer value;
+	gboolean is_string = FALSE;
+	gpointer value = NULL;
 
 	g_return_val_if_fail (E_IS_CONTACT (contact), NULL);
 	g_return_val_if_fail (field_id >= 1 && field_id <= E_CONTACT_LAST_SIMPLE_STRING, NULL);
 
-	value = e_contact_get (contact, field_id);
+	if (field_info [field_id].t & E_CONTACT_FIELD_TYPE_STRING)
+		is_string = TRUE;
 
-	g_object_weak_ref (G_OBJECT (contact), free_const_data, value);
+	if (is_string)
+		value = contact->priv->cached_strings[field_id];
+
+	if (!value) {
+		value = e_contact_get (contact, field_id);
+		if (is_string && value)
+			contact->priv->cached_strings[field_id] = value;
+
+		if (value)
+			g_object_weak_ref (G_OBJECT (contact), free_const_data, value);
+	}
 
 	return value;
 }
@@ -1411,6 +1464,10 @@ e_contact_set (EContact *contact, EContactField field_id, gpointer value)
 	g_return_if_fail (contact && E_IS_CONTACT (contact));
 	g_return_if_fail (field_id >= 1 && field_id <= E_CONTACT_FIELD_LAST);
 
+	/* set the cached slot to NULL so we'll re-get the new string
+	   if e_contact_get_const is called again */
+	contact->priv->cached_strings[field_id] = NULL;
+
 	g_object_set (contact,
 		      e_contact_field_name (field_id), value,
 		      NULL);
