@@ -68,14 +68,14 @@ e_vcard_dispose (GObject *object)
 {
 	EVCard *evc = E_VCARD (object);
 
-	if (!evc->priv)
-		return;
+	if (evc->priv) {
 
-	g_list_foreach (evc->priv->attributes, (GFunc)e_vcard_attribute_free, NULL);
-	g_list_free (evc->priv->attributes);
+		g_list_foreach (evc->priv->attributes, (GFunc)e_vcard_attribute_free, NULL);
+		g_list_free (evc->priv->attributes);
 
-	g_free (evc->priv);
-	evc->priv = NULL;
+		g_free (evc->priv);
+		evc->priv = NULL;
+	}
 
 	if (G_OBJECT_CLASS (parent_class)->dispose)
 		G_OBJECT_CLASS (parent_class)->dispose (object);
@@ -209,6 +209,7 @@ skip_until (char **p, char *s)
 
 		if (s_matches)
 			break;
+		lp = g_utf8_next_char (lp);
 	}
 
 	*p = lp;
@@ -234,19 +235,23 @@ read_attribute_value (EVCardAttribute *attr, char **p, gboolean quoted_printable
 				 * 2 kinds of line folding
 				 */
 			}
-			else if (isalnum(a) && isalnum (b)) {
-				char c;
+			else if (isxdigit(a) && isxdigit (b)) {
+				gunichar c;
 
 				a = tolower (a);
 				b = tolower (b);
 
 				c = (((a>='a'?a-'a'+10:a-'0')&0x0f) << 4)
 					| ((b>='a'?b-'a'+10:b-'0')&0x0f);
-
-				str = g_string_append_c (str, c);
+				
+				str = g_string_append_unichar (str, c);
 			}
-			/* silently consume malformed input, and
-			   continue parsing */
+			else 
+				{
+					str = g_string_append_c (str, a);
+					str = g_string_append_c (str, b);
+				}
+			
 			lp++;
 		}
 		else if (*lp == '\\') {
@@ -358,6 +363,8 @@ read_attribute_params (EVCardAttribute *attr, char **p, gboolean *quoted_printab
 					if (!param->values) {
 						e_vcard_attribute_param_free (param);
 						param = NULL;
+						if (!colon)
+							lp = g_utf8_next_char (lp);
 					}
 				}
 
@@ -377,6 +384,13 @@ read_attribute_params (EVCardAttribute *attr, char **p, gboolean *quoted_printab
 						param_name = "ENCODING";
 						*quoted_printable = TRUE;
 					}
+					/* apple's broken addressbook app outputs naked BASE64
+					   parameters, which aren't even vcard 3.0 compliant. */
+					else if (!g_ascii_strcasecmp (str->str,
+								      "base64")) {
+						param_name = "ENCODING";
+						g_string_assign (str, "b");
+					}
 					else {
 						param_name = "TYPE";
 					}
@@ -390,8 +404,20 @@ read_attribute_params (EVCardAttribute *attr, char **p, gboolean *quoted_printab
 						lp = g_utf8_next_char (lp);
 				}
 				else {
-					/* XXX more here */
-					g_assert_not_reached ();
+					/* we've got an attribute with a truly empty
+					   attribute parameter.  So it's of the form:
+					   
+					   ATTR;[PARAM=value;]*;[PARAM=value;]*:
+
+					   (note the extra ';')
+
+					   the only thing to do here is, well.. nothing.
+					   we skip over the character if it's not a colon,
+					   and the rest is handled for us: We'll either
+					   continue through the loop again if we hit a ';',
+					   or we'll break out correct below if it was a ':' */
+					if (!colon)
+						lp = g_utf8_next_char (lp);
 				}
 			}
 			if (param && !comma) {
@@ -405,6 +431,12 @@ read_attribute_params (EVCardAttribute *attr, char **p, gboolean *quoted_printab
 			g_warning ("invalid character found in parameter spec");
 			g_string_assign (str, "");
 			skip_until (&lp, ":;");
+			
+			if (*lp == '\r') {
+				lp = g_utf8_next_char (lp); /* \n */
+				lp = g_utf8_next_char (lp); /* start of the next line */
+				break;
+			}
 		}
 	}
 
@@ -488,6 +520,8 @@ read_attribute (char **p)
 		/* skip past the ';' */
 		lp = g_utf8_next_char(lp);
 		read_attribute_params (attr, &lp, &is_qp);
+		if (is_qp)
+			attr->encoding = EVC_ENCODING_RAW;
 	}
 	if (*lp == ':') {
 		/* skip past the ':' */
@@ -540,7 +574,7 @@ parse (EVCard *evc, const char *str)
 	}
 	if (attr && !g_ascii_strcasecmp (attr->name, "begin"))
 		e_vcard_attribute_free (attr);
-	else
+	else if (attr)
 		e_vcard_add_attribute (evc, attr);
 
 	while (*p) {
@@ -566,9 +600,11 @@ parse (EVCard *evc, const char *str)
 char*
 e_vcard_escape_string (const char *s)
 {
-	GString *str = g_string_new ("");
+	GString *str;
 	const char *p;
 
+	str = g_string_new ("");
+
 	/* Escape a string as described in RFC2426, section 5 */
 	for (p = s; p && *p; p++) {
 		switch (*p) {
@@ -601,9 +637,13 @@ e_vcard_escape_string (const char *s)
 char*
 e_vcard_unescape_string (const char *s)
 {
-	GString *str = g_string_new ("");
+	GString *str;
 	const char *p;
 
+	g_return_val_if_fail (s != NULL, NULL);
+
+	str = g_string_new ("");
+
 	/* Unescape a string as described in RFC2426, section 5 */
 	for (p = s; *p; p++) {
 		if (*p == '\\') {
@@ -633,6 +673,9 @@ e_vcard_unescape_string (const char *s)
 void
 e_vcard_construct (EVCard *evc, const char *str)
 {
+	g_return_if_fail (E_IS_VCARD (evc));
+	g_return_if_fail (str != NULL);
+
 	if (*str)
 		parse (evc, str);
 }
@@ -648,7 +691,7 @@ e_vcard_new_from_string (const char *str)
 {
 	EVCard *evc;
 
-	g_return_val_if_fail (str, NULL);
+	g_return_val_if_fail (str != NULL, NULL);
 
 	evc = g_object_new (E_TYPE_VCARD, NULL);
 
@@ -685,8 +728,7 @@ e_vcard_to_string_vcard_30 (EVCard *evc)
 		GString *attr_str;
 		int l;
 
-		if (attr->group == NULL &&
-		    !g_ascii_strcasecmp (attr->name, "VERSION"))
+		if (!g_ascii_strcasecmp (attr->name, "VERSION"))
 			continue;
 
 		attr_str = g_string_new ("");
@@ -715,18 +757,18 @@ e_vcard_to_string_vcard_30 (EVCard *evc)
 				for (v = param->values; v; v = v->next) {
 					char *value = v->data;
 					char *p = value;
-					gboolean ws = FALSE;
+					gboolean quotes = FALSE;
 					while (*p) {
-						if (g_unichar_isspace (g_utf8_get_char (p))) {
-							ws = TRUE;
+						if (!g_unichar_isalnum (g_utf8_get_char (p))) {
+							quotes = TRUE;
 							break;
 						}
 						p = g_utf8_next_char (p);
 					}
-					if (ws)
+					if (quotes)
 						attr_str = g_string_append_c (attr_str, '"');
 					attr_str = g_string_append (attr_str, value);
-					if (ws)
+					if (quotes)
 						attr_str = g_string_append_c (attr_str, '"');
 					if (v->next)
 						attr_str = g_string_append_c (attr_str, ',');
@@ -762,13 +804,18 @@ e_vcard_to_string_vcard_30 (EVCard *evc)
 		 */
 		l = 0;
 		do {
-			if (attr_str->len - l > 75) {
+			
+			if ((g_utf8_strlen (attr_str->str, -1) -l) > 75) {
+				char *p;
+
 				l += 75;
-				attr_str = g_string_insert_len (attr_str, l, CRLF " ", sizeof (CRLF " ") - 1);
+				p = g_utf8_offset_to_pointer (attr_str->str, l);
+			
+				attr_str = g_string_insert_len (attr_str, (p - attr_str->str), CRLF " ", sizeof (CRLF " ") - 1);
 			}
 			else
 				break;
-		} while (l < attr_str->len);
+		} while (l < g_utf8_strlen (attr_str->str, -1));
 
 		attr_str = g_string_append (attr_str, CRLF);
 
@@ -784,6 +831,8 @@ e_vcard_to_string_vcard_30 (EVCard *evc)
 char*
 e_vcard_to_string (EVCard *evc, EVCardFormat format)
 {
+	g_return_val_if_fail (E_IS_VCARD (evc), NULL);
+
 	switch (format) {
 	case EVC_FORMAT_VCARD_21:
 		return e_vcard_to_string_vcard_21 (evc);
@@ -802,6 +851,8 @@ e_vcard_dump_structure (EVCard *evc)
 	GList *v;
 	int i;
 
+	g_return_if_fail (E_IS_VCARD (evc));
+
 	printf ("vCard\n");
 	for (a = evc->priv->attributes; a; a = a->next) {
 		GList *p;
@@ -836,7 +887,9 @@ e_vcard_dump_structure (EVCard *evc)
 EVCardAttribute*
 e_vcard_attribute_new (const char *attr_group, const char *attr_name)
 {
-	EVCardAttribute *attr = g_new0 (EVCardAttribute, 1);
+	EVCardAttribute *attr;
+
+	attr = g_new0 (EVCardAttribute, 1);
 
 	attr->group = g_strdup (attr_group);
 	attr->name = g_strdup (attr_name);
@@ -847,6 +900,8 @@ e_vcard_attribute_new (const char *attr_group, const char *attr_name)
 void
 e_vcard_attribute_free (EVCardAttribute *attr)
 {
+	g_return_if_fail (attr != NULL);
+
 	g_free (attr->group);
 	g_free (attr->name);
 
@@ -860,10 +915,14 @@ e_vcard_attribute_free (EVCardAttribute *attr)
 EVCardAttribute*
 e_vcard_attribute_copy (EVCardAttribute *attr)
 {
-	EVCardAttribute *a = e_vcard_attribute_new (e_vcard_attribute_get_group (attr),
-						    e_vcard_attribute_get_name (attr));
+	EVCardAttribute *a;
 	GList *p;
 
+	g_return_val_if_fail (attr != NULL, NULL);
+
+	a = e_vcard_attribute_new (e_vcard_attribute_get_group (attr),
+				   e_vcard_attribute_get_name (attr));
+
 	for (p = attr->values; p; p = p->next)
 		e_vcard_attribute_add_value (a, p->data);
 
@@ -878,6 +937,9 @@ e_vcard_remove_attributes (EVCard *evc, const char *attr_group, const char *attr
 {
 	GList *attr;
 
+	g_return_if_fail (E_IS_VCARD (evc));
+	g_return_if_fail (attr_name != NULL);
+
 	attr = evc->priv->attributes;
 	while (attr) {
 		GList *next_attr;
@@ -885,7 +947,8 @@ e_vcard_remove_attributes (EVCard *evc, const char *attr_group, const char *attr
 
 		next_attr = attr->next;
 
-		if (((!attr_group && !a->group) || !g_ascii_strcasecmp (attr_group, a->group)) &&
+		if (((!attr_group && !a->group) ||
+		     (attr_group && !g_ascii_strcasecmp (attr_group, a->group))) &&
 		    ((!attr_name && !a->name) || !g_ascii_strcasecmp (attr_name, a->name))) {
 
 			/* matches, remove/delete the attribute */
@@ -901,6 +964,9 @@ e_vcard_remove_attributes (EVCard *evc, const char *attr_group, const char *attr
 void
 e_vcard_remove_attribute (EVCard *evc, EVCardAttribute *attr)
 {
+	g_return_if_fail (E_IS_VCARD (evc));
+	g_return_if_fail (attr != NULL);
+
 	evc->priv->attributes = g_list_remove (evc->priv->attributes, attr);
 	e_vcard_attribute_free (attr);
 }
@@ -908,6 +974,9 @@ e_vcard_remove_attribute (EVCard *evc, EVCardAttribute *attr)
 void
 e_vcard_add_attribute (EVCard *evc, EVCardAttribute *attr)
 {
+	g_return_if_fail (E_IS_VCARD (evc));
+	g_return_if_fail (attr != NULL);
+
 	evc->priv->attributes = g_list_append (evc->priv->attributes, attr);
 }
 
@@ -915,6 +984,9 @@ void
 e_vcard_add_attribute_with_value (EVCard *evcard,
 				  EVCardAttribute *attr, const char *value)
 {
+	g_return_if_fail (E_IS_VCARD (evcard));
+	g_return_if_fail (attr != NULL);
+
 	e_vcard_attribute_add_value (attr, value);
 
 	e_vcard_add_attribute (evcard, attr);
@@ -926,6 +998,9 @@ e_vcard_add_attribute_with_values (EVCard *evcard, EVCardAttribute *attr, ...)
 	va_list ap;
 	char *v;
 
+	g_return_if_fail (E_IS_VCARD (evcard));
+	g_return_if_fail (attr != NULL);
+
 	va_start (ap, attr);
 
 	while ((v = va_arg (ap, char*))) {
@@ -940,12 +1015,16 @@ e_vcard_add_attribute_with_values (EVCard *evcard, EVCardAttribute *attr, ...)
 void
 e_vcard_attribute_add_value (EVCardAttribute *attr, const char *value)
 {
+	g_return_if_fail (attr != NULL);
+
 	attr->values = g_list_append (attr->values, g_strdup (value));
 }
 
 void
 e_vcard_attribute_add_value_decoded (EVCardAttribute *attr, const char *value, int len)
 {
+	g_return_if_fail (attr != NULL);
+
 	switch (attr->encoding) {
 	case EVC_ENCODING_RAW:
 		g_warning ("can't add_value_decoded with an attribute using RAW encoding.  you must set the ENCODING parameter first");
@@ -977,6 +1056,8 @@ e_vcard_attribute_add_values (EVCardAttribute *attr,
 	va_list ap;
 	char *v;
 
+	g_return_if_fail (attr != NULL);
+
 	va_start (ap, attr);
 
 	while ((v = va_arg (ap, char*))) {
@@ -995,6 +1076,8 @@ free_gstring (GString *str)
 void
 e_vcard_attribute_remove_values (EVCardAttribute *attr)
 {
+	g_return_if_fail (attr != NULL);
+
 	g_list_foreach (attr->values, (GFunc)g_free, NULL);
 	g_list_free (attr->values);
 	attr->values = NULL;
@@ -1007,6 +1090,8 @@ e_vcard_attribute_remove_values (EVCardAttribute *attr)
 void
 e_vcard_attribute_remove_params (EVCardAttribute *attr)
 {
+	g_return_if_fail (attr != NULL);
+
 	g_list_foreach (attr->params, (GFunc)e_vcard_attribute_param_free, NULL);
 	g_list_free (attr->params);
 	attr->params = NULL;
@@ -1028,6 +1113,8 @@ e_vcard_attribute_param_new (const char *name)
 void
 e_vcard_attribute_param_free (EVCardAttributeParam *param)
 {
+	g_return_if_fail (param != NULL);
+
 	g_free (param->name);
 
 	e_vcard_attribute_param_remove_values (param);
@@ -1038,9 +1125,13 @@ e_vcard_attribute_param_free (EVCardAttributeParam *param)
 EVCardAttributeParam*
 e_vcard_attribute_param_copy (EVCardAttributeParam *param)
 {
-	EVCardAttributeParam *p = e_vcard_attribute_param_new (e_vcard_attribute_param_get_name (param));
+	EVCardAttributeParam *p;
 	GList *l;
 
+	g_return_val_if_fail (param != NULL, NULL);
+
+	p = e_vcard_attribute_param_new (e_vcard_attribute_param_get_name (param));
+
 	for (l = param->values; l; l = l->next) {
 		e_vcard_attribute_param_add_value (p, l->data);
 	}
@@ -1052,6 +1143,9 @@ void
 e_vcard_attribute_add_param (EVCardAttribute *attr,
 			     EVCardAttributeParam *param)
 {
+	g_return_if_fail (attr != NULL);
+	g_return_if_fail (param != NULL);
+
 	attr->params = g_list_append (attr->params, param);
 
 	/* we handle our special encoding stuff here */
@@ -1084,6 +1178,8 @@ void
 e_vcard_attribute_param_add_value (EVCardAttributeParam *param,
 				   const char *value)
 {
+	g_return_if_fail (param != NULL);
+
 	param->values = g_list_append (param->values, g_strdup (value));
 }
 
@@ -1094,6 +1190,8 @@ e_vcard_attribute_param_add_values (EVCardAttributeParam *param,
 	va_list ap;
 	char *v;
 
+	g_return_if_fail (param != NULL);
+
 	va_start (ap, param);
 
 	while ((v = va_arg (ap, char*))) {
@@ -1107,6 +1205,9 @@ void
 e_vcard_attribute_add_param_with_value (EVCardAttribute *attr,
 					EVCardAttributeParam *param, const char *value)
 {
+	g_return_if_fail (attr != NULL);
+	g_return_if_fail (param != NULL);
+
 	e_vcard_attribute_param_add_value (param, value);
 
 	e_vcard_attribute_add_param (attr, param);
@@ -1119,6 +1220,9 @@ e_vcard_attribute_add_param_with_values (EVCardAttribute *attr,
 	va_list ap;
 	char *v;
 
+	g_return_if_fail (attr != NULL);
+	g_return_if_fail (param != NULL);
+
 	va_start (ap, param);
 
 	while ((v = va_arg (ap, char*))) {
@@ -1133,6 +1237,8 @@ e_vcard_attribute_add_param_with_values (EVCardAttribute *attr,
 void
 e_vcard_attribute_param_remove_values (EVCardAttributeParam *param)
 {
+	g_return_if_fail (param != NULL);
+
 	g_list_foreach (param->values, (GFunc)g_free, NULL);
 	g_list_free (param->values);
 	param->values = NULL;
@@ -1141,30 +1247,40 @@ e_vcard_attribute_param_remove_values (EVCardAttributeParam *param)
 GList*
 e_vcard_get_attributes (EVCard *evcard)
 {
+	g_return_val_if_fail (E_IS_VCARD (evcard), NULL);
+
 	return evcard->priv->attributes;
 }
 
 const char*
 e_vcard_attribute_get_group (EVCardAttribute *attr)
 {
+	g_return_val_if_fail (attr != NULL, NULL);
+
 	return attr->group;
 }
 
 const char*
 e_vcard_attribute_get_name (EVCardAttribute *attr)
 {
+	g_return_val_if_fail (attr != NULL, NULL);
+
 	return attr->name;
 }
 
 GList*
 e_vcard_attribute_get_values (EVCardAttribute *attr)
 {
+	g_return_val_if_fail (attr != NULL, NULL);
+
 	return attr->values;
 }
 
 GList*
 e_vcard_attribute_get_values_decoded (EVCardAttribute *attr)
 {
+	g_return_val_if_fail (attr != NULL, NULL);
+
 	if (!attr->decoded_values) {
 		GList *l;
 		switch (attr->encoding) {
@@ -1192,6 +1308,8 @@ e_vcard_attribute_get_values_decoded (EVCardAttribute *attr)
 gboolean
 e_vcard_attribute_is_single_valued (EVCardAttribute *attr)
 {
+	g_return_val_if_fail (attr != NULL, FALSE);
+
 	if (attr->values == NULL
 	    || attr->values->next != NULL)
 		return FALSE;
@@ -1202,7 +1320,11 @@ e_vcard_attribute_is_single_valued (EVCardAttribute *attr)
 char*
 e_vcard_attribute_get_value (EVCardAttribute *attr)
 {
-	GList *values = e_vcard_attribute_get_values (attr);
+	GList *values;
+
+	g_return_val_if_fail (attr != NULL, NULL);
+
+	values = e_vcard_attribute_get_values (attr);
 
 	if (!e_vcard_attribute_is_single_valued (attr))
 		g_warning ("e_vcard_attribute_get_value called on multivalued attribute");
@@ -1213,9 +1335,13 @@ e_vcard_attribute_get_value (EVCardAttribute *attr)
 GString*
 e_vcard_attribute_get_value_decoded (EVCardAttribute *attr)
 {
-	GList *values = e_vcard_attribute_get_values_decoded (attr);
+	GList *values;
 	GString *str = NULL;
 
+	g_return_val_if_fail (attr != NULL, NULL);
+
+	values = e_vcard_attribute_get_values_decoded (attr);
+
 	if (!e_vcard_attribute_is_single_valued (attr))
 		g_warning ("e_vcard_attribute_get_value_decoded called on multivalued attribute");
 
@@ -1228,9 +1354,14 @@ e_vcard_attribute_get_value_decoded (EVCardAttribute *attr)
 gboolean
 e_vcard_attribute_has_type (EVCardAttribute *attr, const char *typestr)
 {
-	GList *params = e_vcard_attribute_get_params (attr);
+	GList *params;
 	GList *p;
 
+	g_return_val_if_fail (attr != NULL, FALSE);
+	g_return_val_if_fail (typestr != NULL, FALSE);
+
+	params = e_vcard_attribute_get_params (attr);
+
 	for (p = params; p; p = p->next) {
 		EVCardAttributeParam *param = p->data;
 
@@ -1251,18 +1382,24 @@ e_vcard_attribute_has_type (EVCardAttribute *attr, const char *typestr)
 GList*
 e_vcard_attribute_get_params (EVCardAttribute *attr)
 {
+	g_return_val_if_fail (attr != NULL, NULL);
+
 	return attr->params;
 }
 
 const char*
 e_vcard_attribute_param_get_name (EVCardAttributeParam *param)
 {
+	g_return_val_if_fail (param != NULL, NULL);
+
 	return param->name;
 }
 
 GList*
 e_vcard_attribute_param_get_values (EVCardAttributeParam *param)
 {
+	g_return_val_if_fail (param != NULL, NULL);
+
 	return param->values;
 }
 
@@ -1474,7 +1611,9 @@ _evc_base64_encode_simple (const char *data, size_t len)
 	unsigned char *out;
 	int state = 0, outlen;
 	unsigned int save = 0;
-	
+
+	g_return_val_if_fail (data != NULL, NULL);
+
 	out = g_malloc (len * 4 / 3 + 5);
 	outlen = _evc_base64_encode_close ((unsigned char *)data, len, FALSE,
 				      out, &state, &save);
@@ -1488,6 +1627,8 @@ _evc_base64_decode_simple (char *data, size_t len)
 	int state = 0;
 	unsigned int save = 0;
 
+	g_return_val_if_fail (data != NULL, 0);
+
 	return _evc_base64_decode_step ((unsigned char *)data, len,
 					(unsigned char *)data, &state, &save);
 }
