@@ -655,9 +655,27 @@ e_gw_item_to_cal_component (EGwItem *item, ECalBackendGroupwise *cbgw)
 		e_cal_component_set_priority (comp, &priority);
 
 		/* EGwItem's completed is a boolean */
-		if (e_gw_item_get_completed (item)) 
+		if (e_gw_item_get_completed (item)) {
 			percent = 100;
-		else 
+			t = e_gw_item_get_completed_date (item);
+			if (t) {
+				itt_utc = icaltime_from_string (t);
+				if (!icaltime_get_timezone (itt_utc))
+					icaltime_set_timezone (&itt_utc, icaltimezone_get_utc_timezone());
+				if (default_zone) {
+					itt = icaltime_convert_to_zone (itt_utc, default_zone); 
+					icaltime_set_timezone (&itt, default_zone);
+					e_cal_component_set_completed (comp, &itt);
+				} else 
+					e_cal_component_set_completed (comp, &itt_utc);
+			} else {
+				/* We are setting the completion date as the current time due to
+				   the absence of completion element in the soap interface for posted
+				   tasks */
+				itt = icaltime_today ();
+				e_cal_component_set_completed (comp,&itt);
+			}
+		} else 
 			percent =0;
 		e_cal_component_set_percent (comp, &percent);
 
@@ -766,6 +784,9 @@ e_gw_connection_send_appointment (EGwConnection *cnc, const char *container, ECa
 	default:
 		status = E_GW_CONNECTION_STATUS_INVALID_OBJECT;
 	}	
+	
+	if (status == E_GW_CONNECTION_STATUS_ITEM_ALREADY_ACCEPTED)
+		return E_GW_CONNECTION_STATUS_OK;
 
 	return status;
 }
@@ -775,13 +796,38 @@ e_gw_connection_create_appointment (EGwConnection *cnc, const char *container, E
 {
 	EGwItem *item;
 	EGwConnectionStatus status;
+	icalproperty *icalprop;
+	gboolean move_cal = FALSE;
+	icalcomponent *icalcomp;
+	char *id = NULL;
 
 	g_return_val_if_fail (E_IS_GW_CONNECTION (cnc), E_GW_CONNECTION_STATUS_INVALID_CONNECTION);
 	g_return_val_if_fail (E_IS_CAL_COMPONENT (comp), E_GW_CONNECTION_STATUS_INVALID_OBJECT);
 
+	icalcomp = e_cal_component_get_icalcomponent (comp);
+	icalprop = icalcomponent_get_first_property (icalcomp, ICAL_X_PROPERTY);
+	while (icalprop) {
+		const char *x_name;
+
+		x_name = icalproperty_get_x_name (icalprop);
+		if (!strcmp (x_name, "X-EVOLUTION-MOVE-CALENDAR")) {
+			move_cal = TRUE;
+			break;
+		}
+
+		icalprop = icalcomponent_get_next_property (icalcomp, ICAL_X_PROPERTY);
+	}
+
 	item = e_gw_item_new_from_cal_component (container, cbgw, comp);
 	e_gw_item_set_container_id (item, container);
-	status = e_gw_connection_send_item (cnc, item, id_list);
+	if (!move_cal)
+		status = e_gw_connection_send_item (cnc, item, id_list);
+	else {
+		e_gw_item_set_source (item, "personal");
+		status = e_gw_connection_create_item (cnc, item, &id);
+		*id_list = g_slist_append (*id_list, id);
+	}
+
 	g_object_unref (item);
 
 	return status;
