@@ -50,6 +50,7 @@ struct _ECalBackendGroupwisePrivate {
 	char *username;
 	char *password;
 	char *container_id;
+	int timeout_id;
 	CalMode mode;
 	icaltimezone *default_zone;
 	GHashTable *categories_by_id;
@@ -130,7 +131,7 @@ populate_cache (ECalBackendGroupwise *cbgw)
 		g_mutex_unlock (mutex);
                 return status;
         }
-	status = e_gw_connection_create_cursor (priv->cnc, priv->container_id, "recipients message recipientStatus", NULL, &cursor);
+	status = e_gw_connection_create_cursor (priv->cnc, priv->container_id, "recipients message recipientStatus default", NULL, &cursor);
 	if (status != E_GW_CONNECTION_STATUS_OK) {
 		e_cal_backend_groupwise_notify_error_code (cbgw, status);
 		g_mutex_unlock (mutex);
@@ -179,6 +180,9 @@ get_deltas (gpointer handle)
         EGwConnectionStatus status; 
 	GSList *deletes = NULL, *updates = NULL, *adds = NULL, *l;
         
+	if (!handle)
+		return FALSE;
+	
 	cbgw = (ECalBackendGroupwise *) handle;
  	cnc = cbgw->priv->cnc; 
  	cache = cbgw->priv->cache; 
@@ -329,25 +333,31 @@ connect_to_server (ECalBackendGroupwise *cbgw)
 					e_cal_backend_notify_error (E_CAL_BACKEND (cbgw), _("Could not create cache file"));
 					return GNOME_Evolution_Calendar_OtherError;
 				}
+			
+			/* read the default timezone*/
+			priv->default_zone = e_cal_backend_cache_get_default_timezone (priv->cache);
 
 			/* Clear the cache before populating it */
 			e_file_cache_clean (E_FILE_CACHE (priv->cache));
-				
+
+			if (priv->default_zone)
+				e_cal_backend_cache_put_default_timezone (priv->cache, priv->default_zone);
+	
 			/* Populate the cache for the first time.*/
 			/* start a timed polling thread set to 10 minutes*/
 			cnc_status = populate_cache (cbgw);
 			if (cnc_status != E_GW_CONNECTION_STATUS_OK) {
 				g_object_unref (priv->cnc);
+				g_object_unref (priv->cache);
 				priv->cnc = NULL;
+				priv->cache = NULL;
 				g_warning (G_STRLOC ": Could not populate the cache");
 				return GNOME_Evolution_Calendar_PermissionDenied;
 			} else {
 				g_object_ref (priv->cnc);
 				g_object_ref (priv->cache);
-				g_timeout_add (CACHE_REFRESH_INTERVAL, (GSourceFunc) get_deltas, (gpointer) cbgw);
+				priv->timeout_id = g_timeout_add (CACHE_REFRESH_INTERVAL, (GSourceFunc) get_deltas, (gpointer) cbgw);
 				priv->mode = CAL_MODE_REMOTE;
-				/* read the default timezone*/
-				priv->default_zone = e_cal_backend_cache_get_default_timezone (priv->cache);
 				return GNOME_Evolution_Calendar_Success;
 			}
 		} else {
@@ -432,6 +442,9 @@ e_cal_backend_groupwise_finalize (GObject *object)
 		priv->user_email = NULL;
 	}
 
+	if (priv->timeout_id) 
+		g_source_remove (priv->timeout_id);
+	
 	g_free (priv);
 	cbgw->priv = NULL;
 
@@ -499,11 +512,13 @@ e_cal_backend_groupwise_get_alarm_email_address (ECalBackendSync *backend, EData
 static ECalBackendSyncStatus
 e_cal_backend_groupwise_get_static_capabilities (ECalBackendSync *backend, EDataCal *cal, char **capabilities)
 {
-	*capabilities = g_strdup (CAL_STATIC_CAPABILITY_NO_EMAIL_ALARMS "," \
-				  CAL_STATIC_CAPABILITY_ONE_ALARM_ONLY "," \
-				  CAL_STATIC_CAPABILITY_REMOVE_ALARMS ","   \
-	                          CAL_STATIC_CAPABILITY_NO_THISANDPRIOR "," \
-				  CAL_STATIC_CAPABILITY_NO_THISANDFUTURE "," \
+	*capabilities = g_strdup (CAL_STATIC_CAPABILITY_NO_EMAIL_ALARMS ","
+				  CAL_STATIC_CAPABILITY_ONE_ALARM_ONLY ","
+				  CAL_STATIC_CAPABILITY_REMOVE_ALARMS ","
+	                          CAL_STATIC_CAPABILITY_NO_THISANDPRIOR ","
+				  CAL_STATIC_CAPABILITY_NO_THISANDFUTURE ","
+				  CAL_STATIC_CAPABILITY_NO_CONV_TO_ASSIGN_TASK ","
+				  CAL_STATIC_CAPABILITY_NO_CONV_TO_RECUR ","
 				  CAL_STATIC_CAPABILITY_SAVE_SCHEDULES);
 
 	return GNOME_Evolution_Calendar_Success;
@@ -1300,7 +1315,7 @@ e_cal_backend_groupwise_remove_object (ECalBackendSync *backend, EDataCal *cal,
 static ECalBackendSyncStatus
 receive_object (ECalBackendGroupwise *cbgw, EDataCal *cal, icalcomponent *icalcomp)
 {
-	ECalComponent *comp;
+	ECalComponent *comp, *modif_comp = NULL;
 	ECalBackendGroupwisePrivate *priv;
 	icalproperty_method method;
 	EGwConnectionStatus status;
@@ -1314,6 +1329,9 @@ receive_object (ECalBackendGroupwise *cbgw, EDataCal *cal, icalcomponent *icalco
 	
 	status = e_gw_connection_send_appointment (priv->cnc, priv->container_id, comp, method, &remove);
 
+	if (status == E_GW_CONNECTION_STATUS_OK && !modif_comp)
+		return GNOME_Evolution_Calendar_Success;
+
 	/* update the cache */
 	if (status == E_GW_CONNECTION_STATUS_OK) {
 		if (remove) {
@@ -1506,6 +1524,7 @@ e_cal_backend_groupwise_init (ECalBackendGroupwise *cbgw, ECalBackendGroupwiseCl
 
 	priv->categories_by_id = g_hash_table_new_full (g_str_hash, g_str_equal, g_free, g_free);
 	priv->categories_by_name = g_hash_table_new_full (g_str_hash, g_str_equal, g_free, g_free);
+	priv->timeout_id = 0;
 
 	/* create the mutex for thread safety */
 	priv->mutex = g_mutex_new ();
