@@ -4,7 +4,7 @@
  * Copyright (c) 1996-2002
  *	Sleepycat Software.  All rights reserved.
  *
- * $Id: mutex.h,v 1.1.1.1.14.2 2004/10/11 18:49:05 hansp Exp $
+ * $Id: mutex.h,v 1.1.1.1.14.3 2004/11/01 18:50:40 hansp Exp $
  */
 
 #ifndef _DB_MUTEX_H_
@@ -728,31 +728,6 @@ typedef unsigned char tsl_t;
 #endif
 #endif
 
-/*********************************************************************
- * amd64/gcc assembly.
- *********************************************************************/
-#ifdef HAVE_MUTEX_X86_64_GCC_ASSEMBLY
-typedef unsigned char tsl_t;
-
-#ifdef LOAD_ACTUAL_MUTEX_CODE
-/*
- * For gcc/amd64, 0 is clear, 1 is set.
- */
-#define  MUTEX_SET(tsl) ({           \
-	register tsl_t *__l = (tsl);          \
-	int __r;              \
-	asm volatile("mov $1,%%rax; lock; xchgb %1,%%al; xor $1,%%rax"\
-		: "=&a" (__r), "=m" (*__l)          \
-		: "1" (*__l)            \
-		);                \
-	__r & 1;              \
-})
-
-#define  MUTEX_UNSET(tsl)  (*(tsl) = 0)
-#define  MUTEX_INIT(tsl)   MUTEX_UNSET(tsl)
-#endif
-#endif
-
 /*********************************************************************
  * x86_64/gcc assembly.
  *********************************************************************/
