@@ -21,6 +21,7 @@
 
 #include <config.h>
 #include <string.h>
+#include <unistd.h>
 #include "e-file-cache.h"
 #include "e-util.h"
 #include "e-xml-hash-utils.h"
@@ -52,7 +53,8 @@ e_file_cache_set_property (GObject *object, guint property_id, const GValue *val
 	switch (property_id) {
 	case PROP_FILENAME :
 		/* make sure the directory for the cache exists */
-		dirname = g_path_get_dirname ((const char *) g_value_get_string (value));
+		priv->filename = g_strdup ( g_value_get_string (value));
+		dirname = g_path_get_dirname (priv->filename);
 		result = e_util_mkdir_hier (dirname, 0700);
 		g_free (dirname);
 		if (result != 0)
@@ -264,7 +266,7 @@ add_key_to_list (const char *key, const char *value, gpointer user_data)
 {
 	GList **keys = user_data;
 
-	*keys = g_list_append (*keys, key);
+	*keys = g_list_append (*keys, (char *) key);
 }
 
 /**
@@ -342,7 +344,7 @@ add_object_to_list (const char *key, const char *value, gpointer user_data)
 {
 	GSList **list = user_data;
 
-	*list = g_slist_prepend (*list, value);
+	*list = g_slist_prepend (*list, (char *) value);
 }
 
 /**
