@@ -42,6 +42,8 @@ struct _EGwItemPrivate {
 	char *start_date;
 	char *end_date;
 	char *due_date;
+	char *completed_date;
+	char *source;
 	gboolean completed;
 	gboolean is_allday_event;
 	char *subject;
@@ -276,7 +278,13 @@ e_gw_item_dispose (GObject *object)
 		if (priv->category_name) {
 			g_free (priv->category_name);
 			priv->category_name = NULL;
+		} 
+		
+		if (priv->source) {
+			g_free (priv->source);
+			priv->source = NULL;
 		}
+
 		free_changes (priv->additions);
 		free_changes (priv->deletions);
 		free_changes (priv->updates);
@@ -328,6 +336,7 @@ e_gw_item_init (EGwItem *item, EGwItemClass *klass)
 	priv->start_date = NULL;
 	priv->end_date = NULL; 
 	priv->due_date = NULL; 
+	priv->completed_date = NULL;
 	priv->trigger = 0;
 	priv->recipient_list = NULL;
 	priv->organizer = NULL;
@@ -427,6 +436,9 @@ set_recipient_list_from_soap_parameter (EGwItem *item, SoupSoapParameter *param)
 		 to none. */
 		subparam = soup_soap_parameter_get_first_child_by_name (param_recipient, "recipientStatus");
                 if (subparam) {
+       			char *formatted_date, *value; 
+			SoupSoapParameter *temp_param ;
+
                         if (soup_soap_parameter_get_first_child_by_name (subparam, "deleted"))
 				recipient->status = E_GW_ITEM_STAT_DECLINED;
 			if (soup_soap_parameter_get_first_child_by_name (subparam, "declined"))
@@ -435,6 +447,15 @@ set_recipient_list_from_soap_parameter (EGwItem *item, SoupSoapParameter *param)
 				recipient->status = E_GW_ITEM_STAT_ACCEPTED;
 			else 	
 				recipient->status = E_GW_ITEM_STAT_NONE;
+			
+			temp_param = soup_soap_parameter_get_first_child_by_name (subparam, "completed");
+			if (temp_param) {
+				value = soup_soap_parameter_get_string_value (temp_param);
+				formatted_date = e_gw_connection_format_date_string (value);
+				e_gw_item_set_completed_date (item, formatted_date);
+				g_free (value);
+				g_free (formatted_date);
+			}
                 }
 		else {
 			/* if recipientStatus is not provided, use the
@@ -1645,6 +1666,24 @@ e_gw_item_set_due_date (EGwItem *item, const char *new_date)
 	item->priv->due_date = g_strdup (new_date);
 }
 
+char *
+e_gw_item_get_completed_date (EGwItem *item)
+{
+	g_return_val_if_fail (E_IS_GW_ITEM (item), NULL);
+
+	return item->priv->completed_date;
+}
+
+void 
+e_gw_item_set_completed_date (EGwItem *item, const char *new_date)
+{
+	g_return_if_fail (E_IS_GW_ITEM (item));
+
+	if (item->priv->completed_date)
+		g_free (item->priv->completed_date);
+	item->priv->completed_date = g_strdup (new_date);
+}
+
 const char *
 e_gw_item_get_subject (EGwItem *item)
 {
@@ -1849,6 +1888,13 @@ e_gw_item_set_trigger (EGwItem *item, int trigger)
 	item->priv->trigger = trigger;
 }
 
+void
+e_gw_item_set_source (EGwItem *item, char *source)
+{
+	g_return_if_fail (E_IS_GW_ITEM (item));
+	item->priv->source = g_strdup (source);
+}
+
 static void
 add_distribution_to_soap_message (EGwItemOrganizer *organizer, GSList *recipient_list, SoupSoapMessage *msg)
 {
@@ -1900,7 +1946,7 @@ add_distribution_to_soap_message (EGwItemOrganizer *organizer, GSList *recipient
 	soup_soap_message_end_element (msg);
 }
 
-void 
+static void 
 e_gw_item_set_calendar_item_elements (EGwItem *item, SoupSoapMessage *msg)
 {
 	EGwItemPrivate *priv = item->priv;
@@ -1910,6 +1956,8 @@ e_gw_item_set_calendar_item_elements (EGwItem *item, SoupSoapMessage *msg)
 		e_gw_message_write_string_parameter (msg, "id", NULL, priv->id);
 	if (priv->container)
 		e_gw_message_write_string_parameter (msg, "container", NULL, priv->container);
+	if (item->priv->source)
+		e_gw_message_write_string_parameter (msg, "source", NULL, item->priv->source );
 
 	if (priv->classification)
 		e_gw_message_write_string_parameter (msg, "class", NULL, priv->classification);
@@ -1925,7 +1973,7 @@ e_gw_item_set_calendar_item_elements (EGwItem *item, SoupSoapMessage *msg)
 		char *str;
 		
 		str = soup_base64_encode (priv->message, strlen (priv->message));
-		dtstring = g_strdup_printf ("%d", strlen (str));
+		dtstring = g_strdup_printf ("%d", (int)strlen (str));
 		soup_soap_message_add_attribute (msg, "length", dtstring, NULL, NULL);
 		g_free (dtstring);
 		soup_soap_message_write_string (msg, str);
@@ -2186,7 +2234,7 @@ append_event_changes_to_soap_message (EGwItem *item, SoupSoapMessage *msg, int c
 			char *str, *message;
 
 			str = soup_base64_encode (priv->message, strlen (priv->message));
-			message = g_strdup_printf ("%d", strlen (str));
+			message = g_strdup_printf ("%d", (int)strlen (str));
 			soup_soap_message_add_attribute (msg, "length", message, NULL, NULL);
 			g_free (message);
 			soup_soap_message_write_string (msg, str);
